@@ -11,7 +11,6 @@ from tempfile import NamedTemporaryFile
 from pluggy import HookimplMarker, PluginManager
 import py
 from devpi_common.types import cached_property
-from distutils.util import strtobool
 from functools import partial
 from .log import threadlog
 from . import fileutil
@@ -35,6 +34,16 @@ DEFAULT_ARGON2_PARALLELISM = 8
 DEFAULT_ARGON2_TIME_COST = 16
 
 
+def strtobool(val):
+    val = val.lower()
+    if val in ('y', 'yes', 't', 'true', 'on', '1'):
+        return True
+    elif val in ('n', 'no', 'f', 'false', 'off', '0'):
+        return False
+    else:
+        raise ValueError(f"invalid truth value {val!r}")
+
+
 def get_pluginmanager(load_entrypoints=True):
     pm = PluginManager("devpiserver")
     # support old plugins, but emit deprecation warnings
@@ -353,7 +362,7 @@ def add_permission_options(parser, pluginmanager):
              "and their indices. The root user can do anything. When this "
              "option is set, only the specified users/groups can create "
              "and modify users and indices. You have to add root "
-             "explicitely if wanted.")
+             "explicitly if wanted.")
 
 
 def addoptions(parser, pluginmanager):
