@@ -474,13 +474,17 @@ class Importer:
                 bases = indexconfig.pop('bases', None)
                 stage = None
                 if stagename == "root/pypi":
+                    # see if the stage was already created
                     stage = self.xom.model.getstage(stagename)
-                    if stage is not None:
-                        stage.modify(**indexconfig)
-                    elif self.xom.config.no_root_pypi:
+                    if self.xom.config.no_root_pypi:
                         continue
                 if stage is None:
-                    stage = user.create_stage(index, **indexconfig)
+                    # we create a stage without any config
+                    stage = user.create_stage(index, type=indexconfig['type'])
+                if stage is not None:
+                    # we use modify with _keep_unknown, so data from
+                    # removed plugins is preserved
+                    stage.modify(**indexconfig, _keep_unknown=True)
                 if "bases" in import_index["indexconfig"]:
                     # we are changing bases directly to allow import with
                     # removed bases without changing the data from the export
@@ -580,7 +584,7 @@ class Importer:
                     requires_python.append(require_python)
                     yanked.append(is_yanked)
                 stage._save_cache_links(
-                    project, links, requires_python, yanked, serial)
+                    project, links, requires_python, yanked, serial, None)
             # devpi-server-2.1 exported with md5 checksums
             if "md5" in mapping:
                 assert "hash_spec" not in mapping
