@@ -7,13 +7,16 @@ from __future__ import unicode_literals
 import aiohttp
 import inspect
 import os
+import os.path
 import asyncio
 import py
+import ssl
 import sys
 import time
 import traceback
 
 from requests import Response, exceptions
+from requests.utils import DEFAULT_CA_BUNDLE_PATH
 from devpi_common.types import cached_property
 from devpi_common.request import new_requests_session
 from .config import parseoptions, get_pluginmanager
@@ -112,7 +115,7 @@ def _main(pluginmanager, argv=None):
     config = parseoptions(pluginmanager, argv)
     args = config.args
 
-    # meta commmands
+    # meta commands
     if args.version:
         print(server_version)
         return 0
@@ -347,12 +350,30 @@ class XOM:
         max_retries = self.config.replica_max_retries
         return self.new_http_session("server", max_retries=max_retries)
 
+    @cached_property
+    def _ssl_context(self):
+        # create an SSLContext object that uses the same CA certs as requests
+        cafile = (
+            os.environ.get("REQUESTS_CA_BUNDLE")
+            or os.environ.get("CURL_CA_BUNDLE")
+            or DEFAULT_CA_BUNDLE_PATH
+        )
+        if cafile and not os.path.exists(cafile):
+            threadlog.warning(
+                "Could not find a suitable TLS CA certificate bundle, invalid path: %s",
+                cafile
+            )
+            cafile = None
+
+        return ssl.create_default_context(cafile=cafile)
+
     def _close_sessions(self):
         self._httpsession.close()
 
     async def async_httpget(self, url, allow_redirects, timeout=None, extra_headers=None):
         timeout = aiohttp.ClientTimeout(total=timeout)
-        async with aiohttp.ClientSession(timeout=timeout, trust_env=True) as session:
+        connector = aiohttp.TCPConnector(ssl=self._ssl_context)
+        async with aiohttp.ClientSession(timeout=timeout, connector=connector, trust_env=True) as session:
             async with session.get(
                 url, allow_redirects=allow_redirects, headers=extra_headers
             ) as response:
@@ -421,7 +442,6 @@ class XOM:
         from devpi_server.view_auth import DevpiSecurityPolicy
         from devpi_server.views import ContentTypePredicate
         from devpi_server.views import route_url, INSTALLER_USER_AGENT
-        from pkg_resources import get_distribution
         from pyramid.config import Configurator
         from pyramid.viewderivers import INGRESS
         log = self.log
@@ -430,8 +450,7 @@ class XOM:
         pyramid_config = Configurator(root_factory='devpi_server.view_auth.RootFactory')
         pyramid_config.set_security_policy(DevpiSecurityPolicy(self))
 
-        version_info = [
-            ("devpi-server", get_distribution("devpi_server").version)]
+        version_info = [("devpi-server", server_version)]
         for plug, distinfo in self.config.pluginmanager.list_plugin_distinfo():
             key = (distinfo.project_name, distinfo.version)
             if key not in version_info:
@@ -502,11 +521,11 @@ class XOM:
             "installer_simple",
             "/{user:[^+/]+}/{index:[^+/]+/?}",
             header="User-Agent:" + INSTALLER_USER_AGENT,
-            accept="text/html", request_method="GET")
+            request_method="GET")
         pyramid_config.add_route(
             "installer_simple_project", "/{user:[^+/]+}/{index}/{project:[^+/]+/?}",
             header="User-Agent:" + INSTALLER_USER_AGENT,
-            accept="text/html", request_method="GET")
+            request_method="GET")
         pyramid_config.add_route(
             "/{user}/{index}/{project}",
             "/{user:[^+/]+}/{index:[^+/]+}/{project:[^+/]+/?}")
@@ -524,6 +543,15 @@ class XOM:
             "/{user:[^+/]+}")
         pyramid_config.add_route("/", "/")
 
+        pyramid_config.add_accept_view_order(
+            'application/vnd.pypi.simple.v1+json')
+        pyramid_config.add_accept_view_order(
+            'application/json',
+            weighs_more_than='application/vnd.pypi.simple.v1+json')
+        pyramid_config.add_accept_view_order(
+            'text/html',
+            weighs_more_than='application/json')
+
         # register tweens for logging, transaction and replication
         pyramid_config.add_tween("devpi_server.views.tween_request_logging")
         pyramid_config.add_tween(
