@@ -103,6 +103,13 @@ class UpstreamNotFoundError(UpstreamError):
     """ If upstream returned a not found error. """
 
 
+class UpstreamNotModified(ModelException):
+    """ If upstream returned a not modified reply. """
+    def __init__(self, msg, *args, etag=None):
+        super().__init__(msg, *args)
+        self.etag = etag
+
+
 class MissesRegistration(ModelException):
     """ A prior registration of release metadata is required. """
 
@@ -397,10 +404,14 @@ class User:
         threadlog.info("created index %s: %s", stage.name, stage.ixconfig)
         return stage
 
+    @cached_property
+    def MirrorStage(self):
+        from .mirror import MirrorStage
+        return MirrorStage
+
     def _getstage(self, indexname, index_type, ixconfig):
         if index_type == "mirror":
-            from .extpypi import PyPIStage
-            cls = PyPIStage
+            cls = self.MirrorStage
         else:
             cls = PrivateStage
         customizer_cls = get_stage_customizer_class(self.xom, index_type)
@@ -582,6 +593,7 @@ class BaseStage(object):
     NotFound = NotFound
     UpstreamError = UpstreamError
     UpstreamNotFoundError = UpstreamNotFoundError
+    UpstreamNotModified = UpstreamNotModified
     MissesRegistration = MissesRegistration
     MissesVersion = MissesVersion
     NonVolatile = NonVolatile
@@ -676,12 +688,8 @@ class BaseStage(object):
         for key, value in list(kwargs.items()):
             if value is RemoveValue:
                 ixconfig[key] = kwargs.pop(key)
-        if kwargs:
-            raise InvalidIndexconfig(
-                ["indexconfig got unexpected keyword arguments: %s"
-                 % ", ".join("%s=%s" % x for x in kwargs.items())])
         ixconfig["type"] = index_type
-        return ixconfig
+        return (ixconfig, kwargs)
 
     @cached_property
     def user(self):
@@ -786,6 +794,7 @@ class BaseStage(object):
             stable=stable)
 
     def get_last_project_change_serial_perstage(self, project, at_serial=None):
+        project = normalize_name(project)
         tx = self.keyfs.tx
         if at_serial is None:
             at_serial = tx.at_serial
@@ -938,9 +947,18 @@ class BaseStage(object):
         if 'type' in kw and self.ixconfig["type"] != kw['type']:
             raise InvalidIndexconfig(
                 ["the 'type' of an index can't be changed"])
-        kw.pop("type", None)
+        kw.pop('type', None)
         kw.pop("projects", None)  # we never modify this from the outside
-        ixconfig = self.get_indexconfig_from_kwargs(**kw)
+        keep_unknown = kw.pop("_keep_unknown", False)
+        (ixconfig, unknown) = self.get_indexconfig_from_kwargs(**kw)
+        if unknown:
+            if keep_unknown:
+                # used to import data when plugins aren't installed anymore
+                ixconfig.update(unknown)
+            else:
+                raise InvalidIndexconfig(
+                    ["indexconfig got unexpected keyword arguments: %s"
+                     % ", ".join("%s=%s" % x for x in unknown.items())])
         # modify user/indexconfig
         with self.user.key.update() as userconfig:
             oldconfig = dict(self.ixconfig)
@@ -1372,7 +1390,7 @@ class PrivateStage(BaseStage):
         try:
             current_index_config = user_config["indexes"][self.index]
         except KeyError:
-            raise KeyError("The index '%s' was not commited yet." % self.index)
+            raise KeyError("The index '%s' was not committed yet." % self.index)
         # if any project is newer than the user config, we are done
         if last_serial >= user_serial:
             return last_serial
@@ -1506,7 +1524,7 @@ class LinkStore:
             assert overwrite is None
             overwrite = sum(x.get('count', 0)
                             for x in link.get_logs() if x.get('what') == 'overwrite')
-            self.remove_links(rel=rel, basename=basename)
+        self.remove_links(rel=rel, basename=basename)
         file_entry = self._create_file_entry(basename, content_or_file)
         if last_modified is not None:
             file_entry.last_modified = last_modified
