@@ -53,6 +53,7 @@ server_version = devpi_server.__version__
 
 
 H_MASTER_UUID = str("X-DEVPI-MASTER-UUID")
+SIMPLE_API_V1_JSON = "application/vnd.pypi.simple.v1+json"
 
 
 API_VERSION = "2"
@@ -68,6 +69,28 @@ INSTALLER_USER_AGENT = r"([^ ]* )*(distribute|setuptools|pip|pex)/.*"
 INSTALLER_USER_AGENT_REGEXP = re.compile(INSTALLER_USER_AGENT)
 
 
+def _select_simple_content_type(request):
+    offers = request.accept.acceptable_offers([
+        "text/html", SIMPLE_API_V1_JSON])
+    if offers:
+        return offers[0][0]
+    return "text_html"
+
+
+def is_simple_json(request):
+    if _select_simple_content_type(request) == SIMPLE_API_V1_JSON:
+        return True
+    return False
+
+
+def is_requested_by_installer(request):
+    return INSTALLER_USER_AGENT_REGEXP.match(request.user_agent or "")
+
+
+def is_simple_json_or_requested_by_installer(request):
+    return is_simple_json(request) or is_requested_by_installer(request)
+
+
 def abort(request, code, body):
     # if no Accept header is set, then force */*, otherwise the exception
     # will be returned as text/plain, which causes easy_install/setuptools
@@ -376,6 +399,18 @@ def devpiserver_authcheck_forbidden(request):
         return True
 
 
+def version_in_filename(version, filename):
+    if version is None:
+        # no version set, so skip check
+        return True
+    if version in filename:
+        return True
+    # PEP 427 escaped wheels
+    if re.sub(r"[^\w\d.]+", "_", version, re.UNICODE) in filename:
+        return True
+    return False
+
+
 class PyPIView:
     def __init__(self, request):
         self.request = request
@@ -515,17 +550,17 @@ class PyPIView:
         """
         request = self.request
         abort_if_invalid_project(request, request.matchdict["project"])
-        requested_by_installer = INSTALLER_USER_AGENT_REGEXP.match(
-            request.user_agent or "")
-        if requested_by_installer:
+        if is_simple_json_or_requested_by_installer(request):
             # for performance reasons we return results directly for
             # known installers
             return self.simple_list_project()
-        return HTTPFound(location=self.request.route_url(
+        response = HTTPFound(location=self.request.route_url(
             "/{user}/{index}/+simple/{project}/",
             user=self.context.username,
             index=self.context.index,
             project=self.context.project))
+        response.vary = set(["Accept", "User-Agent"])
+        return response
 
     @view_config(route_name="/{user}/{index}/+simple/{project}/")
     def simple_list_project(self):
@@ -534,8 +569,7 @@ class PyPIView:
         project = self.context.project
         # we only serve absolute links so we don't care about the route's slash
         stage = self.context.stage
-        requested_by_installer = INSTALLER_USER_AGENT_REGEXP.match(
-            request.user_agent or "")
+        requested_by_installer = is_simple_json_or_requested_by_installer(request)
         try:
             result = stage.get_simplelinks(project, sorted_links=not requested_by_installer)
         except stage.UpstreamError as e:
@@ -554,15 +588,42 @@ class PyPIView:
             whitelist_info = stage.get_mirror_whitelist_info(project)
             embed_form = whitelist_info['has_mirror_base']
             blocked_index = whitelist_info['blocked_by_mirror_whitelist']
-        app_iter = buffered_iterator(self._simple_list_project(
-            stage, project, result, embed_form, blocked_index))
-        response = Response(app_iter=app_iter)
+        content_type = _select_simple_content_type(self.request)
+        if content_type == SIMPLE_API_V1_JSON:
+            app_iter = self._simple_list_project_json_v1(
+                stage, project, result, embed_form, blocked_index)
+        else:
+            app_iter = self._simple_list_project(
+                stage, project, result, embed_form, blocked_index)
+        response = Response(
+            app_iter=buffered_iterator(app_iter),
+            content_type=content_type,
+            vary=set(["Accept", "User-Agent"]))
         if stage.ixconfig['type'] == 'mirror':
             serial = stage.key_projsimplelinks(project).get().get("serial")
-            if serial > 0:
+            if serial is not None and serial > 0:
                 response.headers[str("X-PYPI-LAST-SERIAL")] = str(serial)
         return response
 
+    def _makeurl_factory(self):
+        if self._use_absolute_urls:
+            # for joinpath we need the root url
+            application_url = self.request.application_url
+            if not application_url.endswith("/"):
+                application_url = application_url + "/"
+            url = URL(application_url)
+
+            def make_url(href):
+                return url.joinpath(href)
+        else:
+            # for relpath we need the path of the current page
+            url = URL(self.request.path_info)
+
+            def make_url(href):
+                return URL(url.relpath("/" + href))
+
+        return make_url
+
     def _simple_list_project(self, stage, project, result, embed_form, blocked_index):
         title = "%s: links for %s" % (stage.name, project)
         yield ("<!DOCTYPE html><html><head><title>%s</title></head><body><h1>%s</h1>\n" %
@@ -577,41 +638,53 @@ class PyPIView:
                    "<strong>%s</strong> are included.</p>"
                    % blocked_index).encode('utf-8')
 
-        if self._use_absolute_urls:
-            # for joinpath we need the root url
-            application_url = self.request.application_url
-            if not application_url.endswith("/"):
-                application_url = application_url + "/"
-            url = URL(application_url)
-
-            def make_url(href):
-                return url.joinpath(href).url
-        else:
-            # for relpath we need the path of the current page
-            url = URL(self.request.path_info)
-
-            def make_url(href):
-                return url.relpath("/" + href)
+        make_url = self._makeurl_factory()
 
         for key, href, require_python, yanked in result:
             stage = "/".join(href.split("/", 2)[:2])
-            attribs = 'href="%s"' % make_url(href)
+            attribs = 'href="%s"' % make_url(href).url
             if require_python is not None:
                 attribs += ' data-requires-python="%s"' % escape(require_python)
-            if yanked:
-                attribs += ' data-yanked=""'
+            if yanked is not None and yanked is not False:
+                yanked = "" if yanked is True else yanked
+                attribs += ' data-yanked="%s"' % escape(yanked)
             data = dict(stage=stage, attribs=attribs, key=key)
             yield '{stage} <a {attribs}>{key}</a><br/>\n'.format(
                 **data).encode('utf-8')
 
         yield "</body></html>".encode("utf-8")
 
+    def _simple_list_project_json_v1(self, stage, project, result, embed_form, blocked_index):
+        yield (f'{{"meta":{{"api-version":"1.0"}},"name":"{project}","files":[').encode("utf-8")
+
+        make_url = self._makeurl_factory()
+
+        first = True
+        for key, href, require_python, yanked in result:
+            url = make_url(href)
+            data = {
+                "filename": key,
+                "url": url.url_nofrag,
+                "hashes": {url.hash_type: url.hash_value} if url.hash_type else {}}
+            if require_python is not None:
+                data["requires-python"] = require_python
+            if yanked is not None and yanked is not False:
+                data["yanked"] = yanked
+            info = json.dumps(data, indent=None, sort_keys=False)
+            if first:
+                yield f'{info}'.encode("utf-8")
+                first = False
+            else:
+                yield f',{info}'.encode("utf-8")
+
+        yield "]}".encode("utf-8")
+
     def _index_refresh_form(self, stage, project):
         url = self.request.route_url(
             "/{user}/{index}/+simple/{project}/refresh",
             user=self.context.username, index=self.context.index,
             project=project)
-        title = "Refresh" if stage.ixconfig["type"] == "mirror" else "Refresh PyPI links"
+        title = "Refresh" if stage.ixconfig["type"] == "mirror" else "Refresh mirror links"
         submit = '<input name="refresh" type="submit" value="%s"/>' % title
         return '<form action="%s" method="post">%s</form>' % (url, submit)
 
@@ -623,16 +696,16 @@ class PyPIView:
         to add a / to the end
         """
         request = self.request
-        requested_by_installer = INSTALLER_USER_AGENT_REGEXP.match(
-            request.user_agent or "")
-        if requested_by_installer:
+        if is_simple_json_or_requested_by_installer(request):
             # for performance reasons we return results directly for
             # known installers
-            return self.simple_list_project()
-        return HTTPFound(location=request.route_url(
+            return self.simple_list_all()
+        response = HTTPFound(location=request.route_url(
             "/{user}/{index}/+simple/",
             user=self.context.username,
             index=self.context.index))
+        response.vary = set(["User-Agent"])
+        return response
 
     @view_config(route_name="/{user}/{index}/+simple/")
     def simple_list_all(self):
@@ -647,13 +720,18 @@ class PyPIView:
             abort(self.request, 502, e.msg)
         # at this point we are sure we can produce the data without
         # depending on remote networks
-        requested_by_installer = INSTALLER_USER_AGENT_REGEXP.match(
-            self.request.user_agent or "")
-        if requested_by_installer:
-            app_iter = buffered_iterator(self._simple_list_all_installer(stage, stage_results))
+        content_type = _select_simple_content_type(self.request)
+        if content_type == SIMPLE_API_V1_JSON:
+            app_iter = self._simple_list_all_json_v1(stage, stage_results)
         else:
-            app_iter = buffered_iterator(self._simple_list_all(stage, stage_results))
-        return Response(app_iter=app_iter)
+            if is_requested_by_installer(self.request):
+                app_iter = self._simple_list_all_installer(stage, stage_results)
+            else:
+                app_iter = self._simple_list_all(stage, stage_results)
+        return Response(
+            app_iter=buffered_iterator(app_iter),
+            content_type=content_type,
+            vary=set(["Accept", "User-Agent"]))
 
     def _simple_list_all(self, stage, stage_results):
         title = "%s: simple list (including inherited indices)" % (stage.name)
@@ -685,6 +763,24 @@ class PyPIView:
                         seen.add(name)
         yield "</body></html>".encode("utf-8")
 
+    def _simple_list_all_json_v1(self, stage, stage_results):
+        yield '{"meta":{"api-version":"1.0"},"projects":['.encode("utf-8")
+        last_index = len(stage_results) - 1
+        seen = set()
+        first = True
+        for index, (stage, names) in enumerate(stage_results):
+            for name in names:
+                if name not in seen:
+                    info = f'{{"name":"{name}"}}'
+                    if first:
+                        yield f'{info}'.encode("utf-8")
+                        first = False
+                    else:
+                        yield f',{info}'.encode("utf-8")
+                    if index != last_index:
+                        seen.add(name)
+        yield ']}'.encode("utf-8")
+
     @view_config(
         route_name="/{user}/{index}/+simple/{project}/refresh", request_method="POST")
     def simple_refresh(self):
@@ -1025,7 +1121,7 @@ class PyPIView:
                 # we only check for release files if version is
                 # contained in the filename because for doczip files
                 # we construct the filename ourselves anyway.
-                if version and version not in content.filename:
+                if not version_in_filename(version, content.filename):
                     abort_submit(
                         request, 400,
                         "filename %r does not contain version %r" % (
@@ -1130,7 +1226,8 @@ class PyPIView:
     #  per-project and version data
     #
 
-    @view_config(route_name="installer_simple")
+    @view_config(route_name="installer_simple", accept="text/html")
+    @view_config(route_name="installer_simple", accept="application/vnd.pypi.simple.v1+json")
     def installer_simple(self):
         """
         When an installer accesses a project without the /+simple part we
@@ -1228,7 +1325,7 @@ class PyPIView:
             relpath = relpath.split("#", 1)[0]
         return relpath
 
-    def _pkgserv(self, entry, url):
+    def _pkgserv(self, entry):
         request = self.request
         if not entry.meta:
             abort(request, 410, "file existed, deleted in later serial")
@@ -1240,6 +1337,13 @@ class PyPIView:
         # getting the stage from context will cause 404 if stage is deleted
         stage = self.context.stage
 
+        # we need to add auth back to the url, as aiohttp doesn't include it
+        # in the response url, unlike requests did.
+        # we do it in _pkgserv now to avoid storing the credentials
+        # in the database and avoid changes in the db when mirror_url changes.
+        mirror_url_auth = getattr(stage, "mirror_url_auth", {})
+        url = URL(entry.url).replace(**mirror_url_auth)
+
         file_exists = entry.file_exists()
         if entry.last_modified is None or not file_exists:
             # We check whether we should serve the file directly
@@ -1286,13 +1390,7 @@ class PyPIView:
         if key is None or not key.exists():
             abort(self.request, 404, "no such file")
         entry = self.xom.filestore.get_file_entry_from_key(key)
-        # we need to add auth back to the url, as aiohttp doesn't include it
-        # in the response url, unlike requests did
-        # we do it in mirror_pkgserv now to avoid storing the credentials
-        # in the database and avoid changes in the db when mirror_url changes
-        mirror_url_auth = getattr(self.context.stage, "mirror_url_auth", {})
-        url = URL(entry.url).replace(**mirror_url_auth)
-        return self._pkgserv(entry, url)
+        return self._pkgserv(entry)
 
     @view_config(route_name="/{user}/{index}/+f/{relpath:.*}")
     def stage_pkgserv(self):
@@ -1300,7 +1398,7 @@ class PyPIView:
         entry = self.xom.filestore.get_file_entry(relpath)
         if entry is None:
             abort(self.request, 404, "no such file")
-        return self._pkgserv(entry, entry.url)
+        return self._pkgserv(entry)
 
     @view_config(route_name="/{user}/{index}/+e/{relpath:.*}",
                  permission="del_entry",
@@ -1318,6 +1416,8 @@ class PyPIView:
         entry = filestore.get_file_entry(relpath)
         if entry is None:
             abort(self.request, 404, "package %r doesn't exist" % relpath)
+        elif not entry.meta:
+            abort(self.request, 410, "file %r existed, deleted in later serial" % relpath)
         try:
             stage.del_entry(entry)
         except stage.NotFound as e:
@@ -1335,6 +1435,11 @@ class PyPIView:
             result['projects'] = sorted(stage.list_projects_perstage())
         apireturn(200, type="indexconfig", result=result)
 
+    @view_config(route_name="/{user}/{index}", accept="application/vnd.pypi.simple.v1+json", request_method="GET")
+    @view_config(route_name="/{user}/{index}/", accept="application/vnd.pypi.simple.v1+json", request_method="GET")
+    def index_pep691_get(self):
+        return self.simple_list_all()
+
     #
     # login and user handling
     #
@@ -1515,6 +1620,10 @@ def iter_cache_remote_file(xom, entry, url):
             tx = None
 
         def set_content():
+            nonlocal entry
+            if entry.readonly:
+                entry = xom.filestore.get_file_entry(
+                    entry.relpath, readonly=False)
             entry.file_set_content(
                 f, r.headers.get("last-modified", None),
                 hash_spec=hash_spec)
@@ -1528,6 +1637,8 @@ def iter_cache_remote_file(xom, entry, url):
 
         if not entry.has_existing_metadata():
             if tx is not None:
+                if not tx.write:
+                    xom.keyfs.restart_as_write_transaction()
                 set_content()
             else:
                 with xom.keyfs.transaction(write=True):
@@ -1540,6 +1651,8 @@ def iter_cache_remote_file(xom, entry, url):
             # it back in place without creating a new serial
             # we need a direct write connection to use the io_file_* methods
             if tx is not None:
+                if not tx.write:
+                    xom.keyfs.restart_as_write_transaction()
                 tx.conn.io_file_set(entry._storepath, f)
                 threadlog.debug(
                     "put missing file back into place: %s", entry._storepath)
@@ -1630,12 +1743,7 @@ def iter_remote_file_replica(xom, entry, url):
 
 
 def iter_fetch_remote_file(xom, entry, url):
-    filestore = xom.filestore
-    keyfs = xom.keyfs
     if not xom.is_replica():
-        if not keyfs.tx.write:
-            keyfs.restart_as_write_transaction()
-        entry = filestore.get_file_entry(entry.relpath, readonly=False)
         yield from iter_cache_remote_file(xom, entry, url)
     else:
         yield from iter_remote_file_replica(xom, entry, url)
