@@ -15,11 +15,11 @@ import time
 from .reqmock import reqmock, patch_reqsessionmock  # noqa
 from bs4 import BeautifulSoup
 from contextlib import closing
-from devpi_server import extpypi
+from devpi_server import mirror
 from devpi_server.config import get_pluginmanager
 from devpi_server.main import XOM, parseoptions
+from devpi_common.validation import normalize_name
 from devpi_common.url import URL
-from devpi_server.extpypi import PyPIStage
 from devpi_server.log import threadlog, thread_clear_log
 from pyramid.authentication import b64encode
 from pyramid.httpexceptions import status_map
@@ -34,7 +34,7 @@ except ImportError:
 @pytest.fixture(scope="session")
 def server_version():
     from devpi_server import __version__
-    from pkg_resources import parse_version
+    from devpi_common.metadata import parse_version
     return parse_version(__version__)
 
 
@@ -226,7 +226,7 @@ def makexom(request, gentmp, httpget, monkeypatch, storage_info):
             plugin[0] if isinstance(plugin, tuple) else plugin
             for plugin in plugins]
         default_plugins = [
-            auth_basic, auth_devpi, extpypi, model, replica, view_auth, views,
+            auth_basic, auth_devpi, mirror, model, replica, view_auth, views,
             storage_info["_test_plugin"]]
         for plugin in default_plugins:
             if plugin not in plugins:
@@ -259,9 +259,6 @@ def makexom(request, gentmp, httpget, monkeypatch, storage_info):
             xom = XOM(config)
         else:
             xom = XOM(config, httpget=httpget)
-            if not request.node.get_closest_marker("nomockprojectsremote"):
-                monkeypatch.setattr(extpypi.PyPIStage, "_get_remote_projects",
-                    lambda self: set())
             add_pypistage_mocks(monkeypatch, httpget)
         # verify storage interface
         with xom.keyfs.get_connection() as conn:
@@ -346,20 +343,34 @@ def httpget(pypiurls):
             class mockresponse:
                 def __init__(xself, url):
                     fakeresponse = self.url2response.get(url)
+                    if isinstance(fakeresponse, list):
+                        if not fakeresponse:
+                            pytest.fail(
+                                f"http_api call to {url} has no further replies")
+                        fakeresponse = fakeresponse.pop(0)
                     if fakeresponse is None:
                         fakeresponse = dict(
                             status_code=404,
                             reason="Not Found")
+                    fakeresponse["headers"] = requests.structures.CaseInsensitiveDict(
+                        fakeresponse.setdefault("headers", {}))
                     xself.__dict__.update(fakeresponse)
                     if "url" not in fakeresponse:
                         xself.url = url
                     xself.allow_redirects = allow_redirects
                     if "content" in fakeresponse:
                         xself.raw = py.io.BytesIO(fakeresponse["content"])
+                    xself.headers.setdefault('content-type', fakeresponse.get(
+                        'content_type', 'text/html'))
+                    if "etag" in fakeresponse:
+                        fakeresponse["headers"]["ETag"] = fakeresponse["etag"]
 
-                def close(self):
+                def close(xself):
                     return
 
+                def json(xself):
+                    return json.loads(xself.text)
+
                 @property
                 def status(xself):
                     return xself.status_code
@@ -377,14 +388,27 @@ def httpget(pypiurls):
                 response=r))
             return r
 
-        def mockresponse(self, mockurl, **kw):
-            kw.setdefault("status_code", 200)
+        def _prepare_kw(self, kw):
+            kw.setdefault("status_code", kw.pop("code", 200))
             kw.setdefault("reason", getattr(
                 status_map.get(kw["status_code"]),
                 "title",
                 "Devpi Mock Error"))
-            log.debug("set mocking response %s %s", mockurl, kw)
-            self.url2response[mockurl] = kw
+
+        def set(self, url, **kw):
+            """ Set a reply for all future uses. """
+            self._prepare_kw(kw)
+            log.debug("set mocking response %s %s", url, kw)
+            self.url2response[url] = kw
+
+        def add(self, url, **kw):
+            """ Add a one time use reply to the url. """
+            self._prepare_kw(kw)
+            log.debug("add mocking response %s %s", url, kw)
+            self.url2response.setdefault(url, []).append(kw)
+
+        def mockresponse(self, url, **kw):
+            self.set(url, **kw)
 
         def mock_simple(self, name, text="", pkgver=None, hash_type=None,
                         pypiserial=10000, remoteurl=None, requires_python=None,
@@ -397,9 +421,13 @@ def httpget(pypiurls):
             headers = kw.setdefault("headers", {})
             if pypiserial is not None:
                 headers["X-PYPI-LAST-SERIAL"] = str(pypiserial)
-            self.mockresponse(
-                URL(remoteurl).joinpath(name).asdir().url,
-                text=text, **kw)
+            kw.setdefault("url", URL(remoteurl).joinpath(name).asdir().url)
+            if "etag" in kw:
+                etag = kw.pop("etag")
+                self.add(text=text, **kw, etag=etag)
+                self.add(text=text, **kw, status_code=304)
+            else:
+                self.mockresponse(text=text, **kw)
             return ret
 
         def _getmd5digest(self, s):
@@ -425,22 +453,38 @@ def pypistage(devpiserver_makepypistage, xom):
 
 
 def add_pypistage_mocks(monkeypatch, httpget):
+    _projects = set()
+
     # add some mocking helpers
-    PyPIStage.url2response = httpget.url2response
+    mirror.MirrorStage.url2response = httpget.url2response
 
     def mock_simple(self, name, text=None, pypiserial=10000, **kw):
-        self.cache_retrieve_times.expire(name)
+        cache_expire = kw.pop("cache_expire", True)
+        if cache_expire:
+            self.cache_retrieve_times.expire(name)
+            self.cache_retrieve_times.release(name)
+        add_to_projects = kw.pop("add_to_projects", True)
+        if add_to_projects:
+            self.mock_simple_projects(
+                _projects.union([name]), cache_expire=cache_expire)
         return self.xom.httpget.mock_simple(
             name, text=text, pypiserial=pypiserial, **kw)
-    monkeypatch.setattr(PyPIStage, "mock_simple", mock_simple, raising=False)
+    monkeypatch.setattr(
+        mirror.MirrorStage, "mock_simple", mock_simple, raising=False)
 
-    def mock_simple_projects(self, projectlist):
-        t = "".join('<a href="%s">%s</a>\n' % (name, name) for name in projectlist)
-        threadlog.debug("patching simple page with: %s" %(t))
+    def mock_simple_projects(self, projectlist, cache_expire=True):
+        if cache_expire:
+            self.cache_projectnames.expire()
+        _projects.clear()
+        _projects.update(projectlist)
+        t = "".join(
+            '<a href="%s">%s</a>\n' % (normalize_name(name), name)
+            for name in projectlist)
+        threadlog.debug("patching simple page with: %s" % t)
         self.xom.httpget.mockresponse(self.mirror_url, code=200, text=t)
-
-    monkeypatch.setattr(PyPIStage, "mock_simple_projects",
-                        mock_simple_projects, raising=False)
+    monkeypatch.setattr(
+        mirror.MirrorStage, "mock_simple_projects",
+        mock_simple_projects, raising=False)
 
     def mock_extfile(self, path, content, **kw):
         headers = {"content-length": len(content),
@@ -449,18 +493,19 @@ def add_pypistage_mocks(monkeypatch, httpget):
         url = URL(self.mirror_url).joinpath(path)
         return self.xom.httpget.mockresponse(
             url.url, content=content, headers=headers, **kw)
-    monkeypatch.setattr(PyPIStage, "mock_extfile", mock_extfile, raising=False)
+    monkeypatch.setattr(
+        mirror.MirrorStage, "mock_extfile", mock_extfile, raising=False)
 
 
 @pytest.fixture
 def pypiurls():
     from devpi_server.main import _pypi_ixconfig_default
 
-    class PyPIURL:
+    class MirrorURL:
         def __init__(self):
             self.simple = _pypi_ixconfig_default['mirror_url']
 
-    return PyPIURL()
+    return MirrorURL()
 
 
 @pytest.fixture
