@@ -65,11 +65,11 @@ class TestFileStore:
         ("Cambiatuscromos-0.1.1alpha.linux-x86_64.exe", "Cambiatuscromos", "0.1.1alpha"),
         ("Aesthete-0.4.2.win32.exe", "Aesthete", "0.4.2"),
         ("DTL-1.0.5.win-amd64.exe", "DTL", "1.0.5"),
-        ("Cheetah-2.2.2-1.x86_64.rpm", "Cheetah", "2.2.2-1"),
-        ("Cheetah-2.2.2-1.src.rpm", "Cheetah", "2.2.2-1"),
-        ("Cheetah-2.2.2-1.x85.rpm", "Cheetah", "2.2.2-1"),
-        ("Cheetah-2.2.2.dev1.x85.rpm", "Cheetah", "2.2.2.dev1"),
-        ("Cheetah-2.2.2.dev1.noarch.rpm", "Cheetah", "2.2.2.dev1"),
+        ("Cheetah-2.2.2-1.x86_64.rpm", "Cheetah", "2.2.2"),
+        ("Cheetah-2.2.2-1.src.rpm", "Cheetah", "2.2.2"),
+        ("Cheetah-2.2.2-1.x85.rpm", "Cheetah", "2.2.2"),
+        ("Cheetah-2.2.2.dev1-1.0.x85.rpm", "Cheetah", "2.2.2.dev1"),
+        ("Cheetah-2.2.2.dev1-1.0.noarch.rpm", "Cheetah", "2.2.2.dev1"),
         ("deferargs.tar.gz", "", ""),
         ("hello-1.0.doc.zip", "hello", "1.0"),
         ("Twisted-12.0.0.win32-py2.7.msi", "Twisted", "12.0.0"),
