@@ -2,7 +2,6 @@ from __future__ import unicode_literals
 
 import os
 import sys
-import pkg_resources
 import py
 import pytest
 import json
@@ -15,6 +14,7 @@ from devpi_server.readonly import get_mutable_deepcopy
 from devpi_common.archive import Archive, zip_dict
 from devpi_common.metadata import Version
 from devpi_common.url import URL
+import importlib.resources
 
 import devpi_server
 
@@ -199,9 +199,6 @@ class TestImportExport:
             def __init__(self, options=()):
                 from devpi_server.main import set_state_version
                 self.exportdir = gentmp()
-                self.testdatadir = py.path.local(
-                    pkg_resources.resource_filename(
-                        'test_devpi_server', 'importexportdata'))
                 self.mapp1 = makemapp()
                 set_state_version(self.mapp1.xom.config)
                 self.options = options
@@ -218,16 +215,17 @@ class TestImportExport:
 
             def import_testdata(self, name, options=()):
                 from devpi_server.importexport import import_
-                path = self.testdatadir.join(name).strpath
-                serverdir = gentmp()
-                argv = [
-                    "devpi-import",
-                    "--no-events",
-                    "--serverdir", serverdir]
-                argv.extend(options)
-                argv.extend(["--storage", storage_info["name"]])
-                argv.append(path)
-                assert import_(argv=argv) == 0
+                with importlib.resources.path(
+                        'test_devpi_server', 'importexportdata') as path:
+                    serverdir = gentmp()
+                    argv = [
+                        "devpi-import",
+                        "--no-events",
+                        "--serverdir", serverdir]
+                    argv.extend(options)
+                    argv.extend(["--storage", storage_info["name"]])
+                    argv.append(path / name)
+                    assert import_(argv=argv) == 0
                 mapp = makemapp(options=["--serverdir", serverdir])
                 return mapp
 
@@ -475,12 +473,12 @@ class TestImportExport:
             stage = mapp2.xom.model.getstage(api.stagename)
             stage.offline = True
             projects = stage.list_projects_perstage()
-            assert projects == {'package'}
+            assert projects == {'package': 'package'}
             links = sorted(get_mutable_deepcopy(
                 stage.get_simplelinks_perstage("package")))
             assert links == [
                 ('package-1.1.zip', 'root/pypi/+f/a66/5a45920422f9d/package-1.1.zip', None, None),
-                ('package-1.2.zip', 'root/pypi/+f/b3a/8e0e1f9ab1bfe/package-1.2.zip', None, True),
+                ('package-1.2.zip', 'root/pypi/+f/b3a/8e0e1f9ab1bfe/package-1.2.zip', None, ""),
                 ('package-2.0.zip', 'root/pypi/+f/35a/9e381b1a27567/package-2.0.zip', '>=3.5', None)]
 
     def test_mirrordata(self, caplog, impexp):
@@ -1012,26 +1010,6 @@ class TestImportExport:
             stage = mapp2.xom.model.getstage(api.stagename)
             assert "uploadtrigger_jenkins" not in stage.ixconfig
 
-    def test_import_fails_if_uploadtrigger_jenkins_set(self, impexp):
-        from devpi_server.model import InvalidIndexconfig
-        mapp1 = impexp.mapp1
-        api = mapp1.create_and_use()
-        (user, index) = api.stagename.split('/')
-        with mapp1.xom.keyfs.transaction(write=True):
-            stage = mapp1.xom.model.getstage(api.stagename)
-            with stage.user.key.update() as userconfig:
-                ixconfig = userconfig["indexes"][index]
-                ixconfig["uploadtrigger_jenkins"] = "foo"
-        with mapp1.xom.keyfs.transaction():
-            stage = mapp1.xom.model.getstage(api.stagename)
-            assert "uploadtrigger_jenkins" in stage.ixconfig
-            assert stage.ixconfig["uploadtrigger_jenkins"] == "foo"
-
-        impexp.export()
-
-        with pytest.raises(InvalidIndexconfig, match="uploadtrigger_jenkins"):
-            impexp.new_import()
-
     def test_plugin_index_config(self, impexp):
         class Plugin:
             @hookimpl
@@ -1058,7 +1036,40 @@ class TestImportExport:
             assert "foo_plugin" in stage.ixconfig
             assert stage.ixconfig["foo_plugin"] == "foo"
 
-    @pytest.mark.nomockprojectsremote
+    def test_missing_plugin_index_config(self, impexp):
+        class Plugin:
+            @hookimpl
+            def devpiserver_indexconfig_defaults(self, index_type):
+                return {"foo_plugin": index_type}
+        mapp1 = impexp.mapp1
+        mapp1.xom.config.pluginmanager.register(Plugin())
+
+        api1 = mapp1.create_and_use()
+        with mapp1.xom.keyfs.transaction():
+            stage1 = mapp1.xom.model.getstage(api1.stagename)
+            assert stage1.ixconfig["foo_plugin"] == "stage"
+
+        mapp1.set_indexconfig_option("foo_plugin", "foo")
+        with mapp1.xom.keyfs.transaction():
+            stage1 = mapp1.xom.model.getstage(api1.stagename)
+            assert "foo_plugin" in stage1.ixconfig
+            assert stage1.ixconfig["foo_plugin"] == "foo"
+
+        mapp1.login("root", "")
+        mapp1.set_indexconfig_option("foo_plugin", "bar", "root/pypi")
+
+        impexp.export()
+
+        # now import without the plugin, the data should be preserved
+        mapp2 = impexp.new_import()
+        with mapp2.xom.keyfs.transaction():
+            stage1 = mapp2.xom.model.getstage(api1.stagename)
+            assert "foo_plugin" in stage1.ixconfig
+            assert stage1.ixconfig["foo_plugin"] == "foo"
+            stage2 = mapp2.xom.model.getstage("root/pypi")
+            assert "foo_plugin" in stage2.ixconfig
+            assert stage2.ixconfig["foo_plugin"] == "bar"
+
     def test_mirror_settings_preserved(self, httpget, impexp, pypiurls):
         mapp1 = impexp.mapp1
         indexconfig = dict(
@@ -1082,7 +1093,6 @@ class TestImportExport:
             mirror_cache_expiry=600,
             projects=[])
 
-    @pytest.mark.nomockprojectsremote
     def test_no_mirror_releases_touched(self, httpget, impexp, pypiurls):
         mapp1 = impexp.mapp1
         indexconfig = dict(
