@@ -142,6 +142,8 @@ def test_get_mirror_whitelist_info_private_package(mapp, monkeypatch, testapp):
             assert info['has_mirror_base'] is False
             assert info['blocked_by_mirror_whitelist'] == "root/pypi"
     mapp.use("root/pypi")
+    mapp.xom.httpget.mockresponse(
+        "https://pypi.org/simple/", text='<a href="pkg1"></a>')
     mapp.xom.httpget.mock_simple("pkg1", text="")
     mapp.get_simple("pkg1")
     with mapp.xom.model.keyfs.transaction(write=False):
@@ -597,7 +599,7 @@ class TestStage:
         register_and_store(stage, "some_xyz-1.0.zip", b"123")
         # provoke error if called
         monkeypatch.setattr(
-            "devpi_server.extpypi.PyPIStage.list_versions_perstage",
+            "devpi_server.mirror.MirrorStage.list_versions_perstage",
             lambda self, project: 0 / 0)
         assert stage.list_versions('some_xyz') == {'1.0'}
 
@@ -637,9 +639,13 @@ class TestStage:
             mirror_url="http://pypi.org/simple", type="mirror"))
         mirror2 = user.create_stage("mirror2", **udict(
             mirror_url="http://example.com/simple", type="mirror"))
+        httpget.mockresponse(
+            "http://pypi.org/simple/", text='<a href="someproject"></a>')
         httpget.mock_simple(
             "someproject", "<a href='someproject-1.1.zip' /a>",
             remoteurl=mirror1.mirror_url)
+        httpget.mockresponse(
+            "http://example.com/simple/", text='<a href="someproject"></a>')
         httpget.mock_simple(
             "someproject", "<a href='someproject-1.1-py2.py3-none-any.whl' /a>",
             remoteurl=mirror2.mirror_url)
@@ -713,9 +719,13 @@ class TestStage:
             mirror_url="http://pypi.org/simple", type="mirror"))
         mirror2 = user.create_stage("mirror2", **udict(
             mirror_url="http://example.com/simple", type="mirror"))
+        httpget.mockresponse(
+            "http://pypi.org/simple/", text='<a href="someproject"></a>')
         httpget.mock_simple(
             "someproject", "<a href='someproject-1.1.zip' /a>",
             remoteurl=mirror1.mirror_url)
+        httpget.mockresponse(
+            "http://example.com/simple/", text='<a href="someproject"></a>')
         httpget.mock_simple(
             "someproject", "<a href='someproject-1.1-py2.py3-none-any.whl' /a>",
             remoteurl=mirror2.mirror_url)
@@ -864,7 +874,7 @@ class TestStage:
         stage.store_doczip("pkg1", "1.0", zip_dict({}))
 
         # simulate a second entry with a slightly different name
-        # (XXX not clear if this test is really neccessary. hpk thinks for
+        # (XXX not clear if this test is really necessary. hpk thinks for
         # exporting state from server<2.1.5 with such a double-entry one
         # needs to install 2.1.5 and export from there anyway, clearing
         # the problem. Then again server<2.3.2 allowed the store_doczip
@@ -1063,7 +1073,7 @@ class TestStage:
         assert version == "1.0"
         with stage.xom.keyfs.transaction():
             assert link.entry.file_get_content() == content
-        # delete, which shouldnt trigger devpiserver_on_upload
+        # delete, which shouldn't trigger devpiserver_on_upload
         with stage.xom.keyfs.transaction(write=True):
             linkstore = stage.get_linkstore_perstage("pkg1", "1.0", readonly=False)
             linkstore.remove_links()
@@ -1130,7 +1140,7 @@ class TestStage:
         with xom.keyfs.transaction(write=True):
             stage = user.create_stage(**udict(
                 index="world", bases=(), type="stage", volatile=True))
-            with pytest.raises(KeyError, match='not commited yet'):
+            with pytest.raises(KeyError, match='not committed yet'):
                 stage.get_last_change_serial_perstage()
         assert current_serial == xom.keyfs.get_current_serial() - 1
         current_serial = xom.keyfs.get_current_serial()
@@ -1220,7 +1230,7 @@ class TestStage:
             user = xom.model.create_user("hello", password="123")
             stage = user.create_stage(index="world", type="stage")
         # get_last_project_change_serial_perstage only works with
-        # commited transactions
+        # committed transactions
         with xom.keyfs.transaction() as tx:
             first_serial = tx.at_serial
             assert stage.list_projects_perstage() == set()
@@ -1577,7 +1587,7 @@ def test_setdefault_indexes(xom, model):
 def test_get_indexconfig_lists(xom, key, value, result):
     stage = PrivateStage(
         xom, "user", "index", {"type": "stage"}, StageCustomizer)
-    kvdict = stage.get_indexconfig_from_kwargs(**{key: value})
+    (kvdict, unknown) = stage.get_indexconfig_from_kwargs(**{key: value})
     assert kvdict[key] == result
 
 
@@ -1677,5 +1687,5 @@ def test_get_indexconfig_values(xom, input, expected):
         with pytest.raises(expected):
             stage.get_indexconfig_from_kwargs(**input)
     else:
-        result = stage.get_indexconfig_from_kwargs(**input)
+        (result, unknown) = stage.get_indexconfig_from_kwargs(**input)
         assert result == expected
